<?php


namespace Schaffen\Financial\IntegrationGcred\Repositories;

use DB;
use Freelabois\LaravelQuickstart\Extendables\Repository;

class IntegrationRepository extends Repository
{
    // protected $model = Route::class;
    // protected $searchableFields = [
    //     ['field' => 'rot_codigo'],
    //     ['field' => 'rot_descricao', 'operator' => 'ilike'],
    // ];
    public function lostContracts($filters){
        $per_page = 10;
        $page = null;
        $data_ini_pagamento = null;
        $data_fim_pagamento = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;

        extract($filters);

        $psi = DB::table('financeiro.pcg_sem_integracao')->pluck('psi_fk_pcg_codigo')->toArray();

       
        $sql = DB::table('financeiro.parcela')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original") ,
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'ppg_fk_par_codigo',
            'ppg_fk_pag_codigo',
            'ppg_valor_parcela',
            'ppg_juros_atraso',
            'ppg_multa',
            'ppg_juros_adicionais',
            'ppg_valor_pago',
            'par_data_alteracao',
            'pag_patrimonial'

        ])

        ->join('financeiro.parcela_pagamento', 'ppg_fk_par_codigo', 'par_codigo')
        ->join('financeiro.pagamento', 'pag_codigo', 'ppg_fk_pag_codigo')
        ->where('par_status', 2) // quitado
        ->whereIn('pag_patrimonial', $psi)

        ->when($data_ini_pagamento, function ($query) use ($data_ini_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date >= '%s'", $data_ini_pagamento));
        })
        ->when($data_fim_pagamento, function ($query) use ($data_fim_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date <= '%s'", $data_fim_pagamento));
        })
        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })->get();
        //->paginate($per_page, ['*'], 'page', $page);

        return $sql;

    }

    public function installmentsData($filters){
        $per_page = 10;
        $page = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;
        $contrato_id = null;
        $pessoa_identificador = null;


        extract($filters);

        $sql = DB::table('financeiro.movimentacao_financeira')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original"),
            DB::raw("COALESCE(con_fk_fil_codigo || '/' || to_char(con_data_digitacao, 'ddmm/yyyy')|| '/' || lpad(con_codigo::text, 6, '0'), mvf_codigo::text) as codigo_contrato"),
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'mvf_status',
            'mvf_sentido',
            'pes_identificador',
            'com_fk_con_codigo',
            'com_fk_mvf_codigo',
            'pro_tipo_impressao',
            DB::raw("CASE
                WHEN pro_tipo_impressao IS NOT NULL THEN (select cor_valor from public.codigos_referencias where cor_codigo = 'PRODUTO.PRO_TIPO_IMPRESSAO' and cor_valor_referencia = pro_tipo_impressao::text)
                WHEN dcd_tipo_documento = 1 THEN 'Desconto de Cheque'
                WHEN dcd_tipo_documento = 2 THEN 'Desconto de Duplicata'
                ELSE 'Sem Contrato'
            END AS tipo_impressao")
        ])
        ->join('financeiro.parcela', 'mvf_codigo', 'par_fk_mvf_codigo')
        ->join('cadastro.pessoa', 'mvf_fk_pes_codigo', 'pes_codigo')
        ->leftjoin('contrato.contrato_mvf', 'mvf_codigo', 'com_fk_mvf_codigo')
        ->leftjoin('contrato.contrato', 'com_fk_con_codigo', 'con_codigo')
        ->leftjoin('contrato.produto', 'con_fk_pro_codigo', 'pro_codigo')
        ->leftjoin('contrato.documentos_compensacao_docto', 'mvf_codigo', 'dcd_fk_mvf_codigo')
        ->whereIn('mvf_status', [0,1]) // aberto/parcialmente quitado
        ->where('mvf_sentido', 'C')
        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })
        ->when($contrato_id, function ($query) use ($contrato_id) {
            return $query->where('com_fk_con_codigo', $contrato_id)
                        ->orWhere('mvf_codigo', $contrato_id);
        })
        ->when($pessoa_identificador, function ($query) use ($pessoa_identificador) {
            return $query->where('pes_identificador', $pessoa_identificador);
        })
        ->groupBy('mvf_codigo', 'pes_identificador', 'con_codigo', 'par_codigo', 'com_fk_con_codigo', 'com_fk_mvf_codigo',
        'dcd_tipo_documento', 'pro_tipo_impressao')
        ->get(); 
        //->paginate($per_page, ['*'], 'page', $page);
        
        return $sql;

    }

    public function installmentsDataReceived($filters){
        $per_page = 10;
        $page = null;
        $data_ini_pagamento = null;
        $data_fim_pagamento = null;
        $data_ini_vencimento = null;
        $data_fim_vencimento = null;

        extract($filters);

        $sql = DB::table('financeiro.parcela')
        ->select([
            'par_codigo' , 
            'par_fk_mvf_codigo',
            DB::raw("to_char(par_data_vencimento_original, 'dd/mm/yyyy') as par_data_vencimento_original") ,
            DB::raw("to_char(pag_data_pagamento, 'dd/mm/yyyy') as pag_data_pagamento") ,
            'par_valor_original',
            'par_valor_atual',
            'par_status',
            'ppg_fk_par_codigo',
            'ppg_valor_parcela',
            'ppg_juros_atraso',
            'ppg_multa',
            'ppg_juros_adicionais',
            'ppg_valor_pago',
            'par_data_alteracao',
            'com_fk_con_codigo',
            'com_fk_mvf_codigo',
             DB::raw("COALESCE(con_fk_fil_codigo || '/' || to_char(con_data_digitacao, 'ddmm/yyyy')|| '/' || lpad(con_codigo::text, 6, '0'), par_fk_mvf_codigo::text) as codigo_contrato") ,
        ])
        ->leftJoin('contrato.contrato_mvf', 'par_fk_mvf_codigo', 'com_fk_mvf_codigo')
        ->leftJoin('contrato.contrato', 'com_fk_con_codigo', 'con_codigo')
        ->join('financeiro.parcela_pagamento', 'ppg_fk_par_codigo', 'par_codigo')
        ->join('financeiro.pagamento', 'ppg_fk_pag_codigo', 'pag_codigo')
        ->where('par_status', '<>', 0) // Quitado / Parcialmente quitado
        
        ->when($data_ini_pagamento, function ($query) use ($data_ini_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date >= '%s'", $data_ini_pagamento));
        })
        ->when($data_fim_pagamento, function ($query) use ($data_fim_pagamento) {
            return $query->whereRaw(sprintf("pag_data_pagamento::date <= '%s'", $data_fim_pagamento));
        })

        ->when($data_ini_vencimento, function ($query) use ($data_ini_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date >= '%s'", $data_ini_vencimento));
        })
        ->when($data_fim_vencimento, function ($query) use ($data_fim_vencimento) {
            return $query->whereRaw(sprintf("par_data_vencimento_original::date <= '%s'", $data_fim_vencimento));
        })->get();
        //->paginate($per_page, ['*'], 'page', $page);
        
        return $sql;

    }

    public function personData($filters){
        $pes_identificador = '';
        extract($filters);
        var_dump(DB::connection()->getConfig());
        $sql = DB::table('cadastro.pessoa')
        ->select([
            'pes_codigo',
            'pes_nome',
            'pes_fk_reg_codigo',
            'pes_fk_seg_codigo',
            'peco_telefone',
            'peco_codigo',
            'peco_contato',
            'peco_email',
            'peco_fk_pes_codigo',
            'peco_padrao',
            'pee_cep',
            'pee_numero',
            'pee_logradouro',
            'pee_bairro',
            'pee_cidade',
            'pee_uf',
            'pee_pais',           
            'pee_complemento',
            'pee_tipo_endereco',
            'pee_tipo_residencia',
            'pee_distancia_km',
            'pes_identificador',
            'pee_padrao'
        ])
        ->leftJoin('cadastro.pessoa_contato as peco', 'peco.peco_fk_pes_codigo', '=', 'cadastro.pessoa.pes_codigo')
        ->leftJoin('cadastro.pessoa_endereco as pee', 'pee.pee_fk_pes_codigo', '=', 'cadastro.pessoa.pes_codigo')
        ->where('cadastro.pessoa.pes_identificador', "$pes_identificador")
        ->orderBy('cadastro.pessoa.pes_codigo')
        ->orderBy('peco.peco_padrao', 'desc')
        ->orderBy('pee.pee_padrao', 'desc')
        ->orderBy('peco.peco_codigo')
        ->orderBy('pee.pee_codigo')
        ->distinct('cadastro.pessoa.pes_codigo')->get();

        return $sql;

    }

    public function contractData($filters){
    
        $per_page = 10;
        $page = null;
        $data_ini = null;
        $data_fim = null;

        extract($filters);

        $sql = DB::table('contrato.v$contrato', 'c')
        ->select([
            'c.con_codigo' , 
            DB::raw("to_char(c.con_data_aprovacao, 'dd/mm/yyyy') as con_data_aprovacao"),
            DB::raw("to_char(c.con_data_cancelamento, 'dd/mm/yyyy') as con_data_cancelamento"),
            DB::raw("to_char(c.con_data_vencimento, 'dd/mm/yyyy') as con_data_vencimento"),
            DB::raw("to_char(c.con_data_vencimento, 'dd') as con_dia_vencimento"),
            DB::raw("to_char(c.con_data_liberacao, 'dd/mm/yyyy') as con_data_liberacao"),
            DB::raw("to_char(c.con_data_liberacao, 'yyyy') as con_ano_liberacao"),
            'cm.com_fk_con_codigo',
            'cm.com_fk_mvf_codigo',
            'c.con_user_insert' ,
            DB::raw("to_char(c.con_date_insert, 'dd/mm/yyyy hh24:mi') as con_date_insert"),
            'c.con_user_update' ,
            DB::raw("to_char(c.con_date_update, 'dd/mm/yyyy hh24:mi') as con_date_update"),
            'c.con_fk_age_codigo' ,
            'c.con_fk_ban_codigo' ,
            'c.con_fk_fil_codigo' ,
            'fil_descricao' ,
            'c.con_fk_pes_codigo' ,
            'cli.pes_nome' ,
            'cli.pes_identificador as pes_identificador_cliente' ,
            'cli.pes_tipo as pes_tipo_cliente',            
            'pe.pee_cep',
            'pe.pee_numero',
            'pe.pee_logradouro',
            'pe.pee_bairro',
            'pe.pee_cidade',
            'pe.pee_uf',
            'pe.pee_pais',           
            'pe.pee_complemento',
            'pe.pee_tipo_residencia',
            'pe.pee_distancia_km',
            'pc.peco_telefone',
            'pc.peco_codigo',
            'pc.peco_contato',
            'pc.peco_email',
            'cli.pes_identificador',
            'c.con_fk_pes_codigo_fun',
            'c.con_fk_pra_codigo',
            'c.con_fk_pro_codigo' ,
            'c.con_fk_sta_codigo' ,
            'sta_descricao' ,
            'sta_cor' ,
            'sta_muda_fase' ,
            'c.con_numero_conta' ,
            'c.con_digito_verificador' ,
            'c.con_numero_vezes' ,
            'c.con_dias_carencia' ,
            'c.con_percentual_comissao_fun' ,
            'c.con_percentual_comissao_prod' ,
            'c.con_tipo' ,
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'CONTRATO.CON_TIPO' and cor_valor_referencia = con_tipo::text) as descricao_tipo") ,
            'c.con_tipo_pagamento' ,
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'CONTRATO.CON_TIPO_PAGAMENTO' and cor_valor_referencia = con_tipo_pagamento::text) as descricao_pagamento"),
            DB::raw("(select cor_valor from public.v\$codigos_referencias where cor_codigo = 'PRODUTO.PRO_TIPO_IMPRESSAO' and cor_valor_referencia = pro_tipo_impressao::text) as descricao_tipo_impressao") ,
            'c.con_usuario_aprovacao' ,
            'c.con_usuario_cancelamento' ,
            'c.con_valor' ,
            'c.con_valor_comissao' ,
            'c.con_valor_total' ,
            'c.con_fase_autenticada' ,
            'c.con_coeficiente' ,
            'c.con_valor_parcela' ,
            'c.con_taxa_juro' ,
            'c.con_taxa_iof' ,
            DB::raw("fil_codigo_filial || '/' || to_char(con_data_digitacao, 'ddmm/yyyy')|| '/' || lpad(con_codigo::text, 6, '0') as codigo_contrato") ,
            DB::raw("to_char(c.con_data_digitacao, 'dd/mm/yyyy') as con_data_digitacao") ,
            'con_fk_pes_codigo_fiador' ,
            'con_fk_pes_codigo_fiador2' ,
            'c.con_nome_vendedor_veiculo',
            'c.con_cpf_vendedor_veiculo' ,
            'c.con_parecer_origem' ,
            'c.con_parecer_superior' ,
            'c.con_fk_mvf_codigo_comissao' ,
            'c.con_numero_beneficio' ,
            'c.con_fk_con_codigo' ,
            'c.con_valor_bem' ,
            'c.con_descricao_bem' ,
            'c.con_reneg_valor' ,
            DB::raw("(select con_codigo from contrato.v\$contrato c1 where c1.con_fk_con_codigo = c.con_codigo) as con_codigo_renegociacao"),
        ])

        ->join('cadastro.v$pessoa as cli', 'cli.pes_codigo', 'c.con_fk_pes_codigo')
        ->leftJoin('cadastro.v$pessoa_endereco as pe', function($join)
            {
                $join
                ->on('pe.pee_fk_pes_codigo', '=', 'cli.pes_codigo')
                ->on('pe.pee_padrao','=', DB::raw('1'));
            }
        )
        ->leftjoin('cadastro.v$pessoa_contato as pc', function($join)
            {
                $join
                ->on('pc.peco_fk_pes_codigo', '=', 'cli.pes_codigo')
                ->on('pc.peco_padrao','=', DB::raw('1'));
            }
        )
        ->join('cadastro.v$filial', 'fil_codigo', 'c.con_fk_fil_codigo')
        ->join('contrato.v$status', 'sta_codigo', 'c.con_fk_sta_codigo')
        ->join('contrato.v$produto', 'pro_codigo', 'c.con_fk_pro_codigo')
        ->join('cadastro.v$pessoa as prom', 'prom.pes_codigo', 'c.con_fk_pes_codigo_prom')
        ->join('contrato.contrato_mvf as cm', 'cm.com_fk_con_codigo', 'c.con_codigo')
        ->when($data_ini, function ($query) use ($data_ini) {
            return $query->whereRaw(sprintf("c.con_data_digitacao::date >= '%s'", $data_ini));
        })
        ->when($data_fim, function ($query) use ($data_fim) {
            return $query->whereRaw(sprintf("c.con_data_digitacao::date <= '%s'", $data_fim));
        })->get();
        //->paginate($per_page, ['*'], 'page', $page);
        return $sql;

    }

    protected function applyCustomFilters()
    {
        extract($this->filters);
    }

}